program native_test

declare integer constant REP = 10
declare integer i, xscale
declare string cmdlin
external long function lib$get_foreign(string)
external sub testint(integer)
external sub testfp(integer)
external sub teststr(integer)

print "Basic:"
call lib$get_foreign(cmdlin)
if len(cmdlin) > 0 then
    xscale = val%(cmdlin)
else
    xscale = 1
end if
for i = 1 to REP
    call testint(xscale)
next i
for i = 1 to REP
    call testfp(xscale)
next i
for i = 1 to REP
    call teststr(xscale)
next i

end program
!
sub testint(integer xscale)

declare integer constant NINT = 10000
declare integer constant N = 1000000
declare integer i, j, nintscale, sum
declare quad t1, t2
external sub printres(quad, quad, integer, integer, string)
external long function sys$gettim(quad)

nintscale = NINT / xscale
call sys$gettim(t1)
for i = 1 to nintscale
    sum = i - 1%
    for j = 1 to N
        sum = ((sum + 1%) * 2% + 1%) / 2%
    next j
    if sum <> (i - 1% + N) then
        print "Integer test error"
        stop
    end if
next i
call sys$gettim(t2)
call printres(t1, t2, nintscale, N, "integer operations")

end sub
!
sub testfp(integer xscale)

declare integer constant NFP = 1000
declare integer constant N = 1000000
declare integer i, j, nfpscale
declare double sum
declare quad t1, t2
external sub printres(quad, quad, integer, integer, string)

nfpscale = NFP / xscale
call sys$gettim(t1)
for i = 1 to nfpscale
    sum = i - 1.0
    for j = 1 to N
        sum = ((sum + 1.0) * 2.0 + 1.0) / 2.0
    next j
    if abs(sum - (i - 1.0 + 1.5 * N)) > 1 then
        print "Floating point test error"
        stop
    end if
next i
call sys$gettim(t2)
call printres(t1, t2, nfpscale, N, "floating point operations")

end sub
!
sub teststr(integer xscale)

declare integer constant NSTR = 100
declare integer constant N = 1000000
declare string constant ALFA = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
declare integer constant VMS_FACTOR = 100
declare integer i, j, nstrscale, ix1, ix2
declare string s, buf
declare quad t1, t2
external sub printres(quad, quad, integer, integer, string)

nstrscale = NSTR / xscale
call sys$gettim(t1)
for i = 1 to nstrscale * VMS_FACTOR
    buf = ""
    for j = 1 to (N / VMS_FACTOR) / 10
        s = ALFA + ALFA
        ix = mod(i + 1 + 10 * (j - 1), len(ALFA))
        buf = buf + mid(s, ix + 1, 1) + mid(s, ix + 2, 2) + mid(s, ix + 4, 3) + mid(s, ix + 7, 4)
    next j
    ix1 = (N / VMS_FACTOR) / 3
    ix2 = 2 * (N / VMS_FACTOR) / 3
    if((len(buf) <> (N / VMS_FACTOR)) or &
       (mid(buf, ix1, 1) <> mid(ALFA, mod(i + ix1, len(ALFA)) + 1, 1)) or &
       (mid(buf, ix2, 1) <> mid(ALFA, mod(i + ix2, len(ALFA)) + 1, 1))) then
        print "String test error"
        stop
    end if
next i
call sys$gettim(t2)
call printres(t1, t2, nstrscale, N / 10, "string operations")

end sub
!
sub printres(quad t1, quad t2, integer n1, integer n2, string ops)

declare double xperf

xperf = n1 * 1.0 * n2 / ((t2 - t1) / 10000000.0)
print using "####.#### million 'E per second", (xperf / 1000000), ops

end sub
